<?php

namespace App\Http\Controllers;

use App\Services\SocksProxy;
use Illuminate\Http\Request;

class ApiController extends Controller
{
    protected $socksProxy;

    // 依赖注入 SocksProxy 类
    public function __construct(SocksProxy $socksProxy)
    {
        $this->socksProxy = $socksProxy;
    }

    /**
     * 通过 SOCKS 代理发送 GET 请求
     *
     * @return \Illuminate\Http\Response
     */
    public function sendGetRequest()
    {
        $url = 'https://httpbin.org/ip';  // 你可以将其替换为你需要请求的 URL
        $headers = [
            'User-Agent' => 'MyApp/1.0',
            'Accept' => 'application/json',
        ];

        try {
            $response = $this->socksProxy->sendRequest($url, 'GET', $headers);
            return response($response);
        } catch (\Exception $e) {
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    /**
     * 通过 SOCKS 代理发送 POST 请求
     *
     * @return \Illuminate\Http\Response
     */
    public function sendPostRequest()
    {
        $url = 'https://httpbin.org/post';  // 你可以将其替换为你需要请求的 URL
        $headers = [
            'User-Agent' => 'MyApp/1.0',
            'Accept' => 'application/json',
        ];
        $body = json_encode([
            'name' => 'John Doe',
            'email' => 'john.doe@example.com',
        ]);

        try {
            $response = $this->socksProxy->sendRequest($url, 'POST', $headers, $body);
            return response($response);
        } catch (\Exception $e) {
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }
}
