<?php

namespace App\Services;

use Exception;

class SocksProxy
{
    private $proxyHost;
    private $proxyPort;

    // 构造函数设置代理的主机和端口，默认值为 localhost 和 1080 端口
    public function __construct($proxyHost = '127.0.0.1', $proxyPort = 1080)
    {
        $this->proxyHost = $proxyHost;
        $this->proxyPort = $proxyPort;
    }

    /**
     * 通过 SOCKS 代理发送 HTTP 或 HTTPS 请求
     *
     * @param string $url 请求的目标 URL
     * @param string $method 请求方法 (GET 或 POST)
     * @param array $headers 自定义请求头
     * @param string $body 请求体 (仅适用于 POST 请求)
     * @return string 返回响应内容
     */
    public function sendRequest($url, $method = 'GET', $headers = [], $body = null)
    {
        // 解析目标 URL
        $parsedUrl = parse_url($url);
        $host = $parsedUrl['host'];
        $path = isset($parsedUrl['path']) ? $parsedUrl['path'] : '/';
        $query = isset($parsedUrl['query']) ? '?' . $parsedUrl['query'] : '';
        $port = isset($parsedUrl['port']) ? $parsedUrl['port'] : 443;

        // 使用 fsockopen() 连接到 SOCKS 代理
        $proxy = fsockopen($this->proxyHost, $this->proxyPort, $errno, $errstr, 30);
        if (!$proxy) {
            throw new Exception("连接 SOCKS 代理失败: $errstr ($errno)");
        }

        // 对于 HTTPS 请求，使用 CONNECT 方法
        if ($port == 443) {
            $request = "CONNECT $host:443 HTTP/1.1\r\n";
            $request .= "Host: $host\r\n";
            $request .= "Connection: close\r\n\r\n";
            fwrite($proxy, $request);

            // 获取响应，确认代理连接是否成功
            $response = '';
            while (!feof($proxy)) {
                $response .= fgets($proxy, 1024);
            }
            if (strpos($response, "200 Connection established") === false) {
                throw new Exception("无法通过 SOCKS 代理建立 HTTPS 连接");
            }
        }

        // 构建 HTTP 请求
        $request = "$method $path$query HTTP/1.1\r\n";
        $request .= "Host: $host\r\n";
        $request .= "Connection: close\r\n";

        // 设置请求头
        foreach ($headers as $key => $value) {
            $request .= "$key: $value\r\n";
        }

        // 如果是 POST 请求，添加请求体数据
        if ($method == 'POST' && $body) {
            $request .= "Content-Length: " . strlen($body) . "\r\n\r\n";
            $request .= $body;
        } else {
            $request .= "\r\n";
        }

        // 发送请求
        fwrite($proxy, $request);

        // 获取响应
        $response = '';
        while (!feof($proxy)) {
            $response .= fgets($proxy, 1024);
        }

        fclose($proxy);
        return $response;
    }
}
